--

with Ada.Finalization;

with Executors;
with Buffers.Mailboxes;

generic
   with package My_Executors is new Executors (<>);

package Thread_Pool_Executors is
   use Ada;
   package Mex renames My_Executors;

   type Thread_Pool_Executor_Type (Pool_Size : Positive) is
     new Finalization.Limited_Controlled and Mex.Executor_Type with private;

   type Any_Thread_Pool_Executor_Access_Type is access all Thread_Pool_Executor_Type'Class;

   overriding procedure Submit
     (Exec      : in out Thread_Pool_Executor_Type;
      Command   : in     Mex.My_Callables.Any_Callable_Access_Type;
      Params    : in     Mex.Parameter_Type;
      My_Future : in     Mex.My_Futures.Any_Future_Access_Type);

private
   procedure Initialize (Tpe : in out Thread_Pool_Executor_Type);

   package My_Buffers is new Buffers (Positive);
   package My_Mailboxes is new My_Buffers.Mailboxes;

   task type Worker_Task_Type is
      entry Fork
        (This   : in Mex.My_Callables.Any_Callable_Access_Type;
         Params : in Mex.Parameter_Type;
         Result : in Mex.My_Futures.Any_Future_Access_Type);
   end Worker_Task_Type;

   type My_Worker_Pool_Task_Array_Type is array (Positive range <>) of Worker_Task_Type;

   type Thread_Pool_Executor_Type
     (Pool_Size : Positive) is new Finalization.Limited_Controlled and
     My_Executors.Executor_Type with
   record
      Controller : My_Mailboxes.Mailbox_Type (Pool_Size);
      Pool       : My_Worker_Pool_Task_Array_Type (Positive'First .. Pool_Size);
   end record;

end Thread_Pool_Executors;
