--
with Buffers.Mailboxes;
with Futures;
with Default_Reusable_Futures;
with Executors.Completion_Services;
with Completion_Pool_Executors;
with Coloured_Images;
with Segment_Callables;
with Segment_Calculations;

package body Segment_Manager is

   Pool_Size : constant Positive := 8; -- 4 <= NOK

   subtype My_Segment_Data_Type is Coloured_Images.Sci.Segment_Record_Type;

   package Segment_Futures is new Futures
    (Data_Type => My_Segment_Data_Type);

   package Reusable_Segment_Futures is new Default_Reusable_Futures
    (My_Futures => Segment_Futures);

   package Segment_Executors is new Executors
     (Parameter_Type => My_Segment_Data_Type,
      Result_Type    => My_Segment_Data_Type,
      My_Futures     => Segment_Futures,
      My_Callables   => Segment_Callables);

   package Segment_Executors_Completion_Services is
      new Segment_Executors.Completion_Services;

   package Segment_Completion_Pools is new Completion_Pool_Executors
     (My_Executors           => Segment_Executors,
      My_Completion_Services => Segment_Executors_Completion_Services);

   type My_Callable_Access_Type is access Segment_Callables.Callable_Type;

   type Segment_Future_Array_Type is
     array (Coloured_Images.Outer_Index_Type,
            Coloured_Images.Outer_Index_Type) of
     aliased Reusable_Segment_Futures.Default_Future_Type;

   My_Workers : Segment_Completion_Pools.Completion_Pool_Executor_Type (Pool_Size);
   Segment_Future_Array : aliased Segment_Future_Array_Type;
   Segment_Future_Access : Segment_Futures.Any_Future_Access_Type;
   Segment_Callable : aliased Segment_Calculations.Segment_Callable_Type;

   My_Picture : access Coloured_Images.Sci.Segmented_Matrix_Type :=
     new Coloured_Images.Sci.Segmented_Matrix_Type;

   task body Manager is separate;

end Segment_Manager;
